
C--------------------------------------------------------------------
C     calculate a density section in an arbitrary plane given by three 
C     points, from a MaxEnt-density file using relative cell-coordinates 
C     -- interpolate the array to the given plane
C--------------------------------------------------------------------
C
      SUBROUTINE MAPXXX(mpix,NPIX,filden,title,
     $  CELL,DELC,ca,cb,cg,sa,sb,sg,as,bs,cs,ga,gb,gg,pi,
     $  P1,P2,P3,imapor,maplx,maply,imappx,imappy)
C
C     *** mpix = max. number of pixels along one cell axis
C     *** NPIX = actual number of pixels along the three cell axes
C     *** DELC(0..NPIX(1)-1, 0.. , 0..) contains the whole cell 3-dim density
C     *** D(1..lx,1..ly) gets the 2-dim map from this subroutine
C     *** ca/cb/cg = cos(alpha/beta/gamma), sa/sb/sg = sin ...
C
      IMPLICIT  none
      integer NPIX(3),mpix
      real*8 DELC(0:mpix,0:mpix,0:mpix),CELL(6)
      real*8 ca,cb,cg,sa,sb,sg,as,bs,cs,ga,gb,gg,pi
      CHARACTER filden*50,filmap*50,title*200,string*80
C
C     *** arrays with the original point and difference vector coordinates
      real*8 P1(3), P2(3), P3(3)
      real*8 D12(3), D13(3)
C     *** arrays for base-transformation of coordinates:
      real*8 AM(3,3), AMI(3,3)
      real*8 UP1(3), UP2(3), UP3(3), NU(3), NP(3), TEMP(3), TEMP2(3)
     $       ,CM123(3)
      integer i

C     *** variables for section calculation
      integer imappx,imappy,imapor
      real*8 deltax,deltay,density,START(3),X(3),UX(3)
      real*8 power,smooth,maplx,maply,wert,densum
      integer iproj,iprojn,ip,ix,iy
      real*8 projstep,projd1,projd2
      logical CR,error
      integer next
 2000 FORMAT(BN,I3)
 2010 FORMAT(BN,A30)
C 2020 FORMAT(BN,I5)
1001  FORMAT(a,'  [default=',f8.5,']')
1002  FORMAT(a,'  [default=',i4,']')

4     write(*,*) 'Please input three corners in relative ',
     $  'cell coordinates'
      print *,'Order of points is:'
      print *,'1. lower left'
      print *,'2. lower right and'
      print *,'3. upper left point of your plane-defining points.'
      print *
      string='1. Lower left corner : x'
      CALL inp(string,P1(1))
      string='1. Lower left corner : y'
      CALL inp(string,P1(2))
      string='1. Lower left corner : z'
      CALL inp(string,P1(3))
      string='2. Lower right corner : x'
      CALL inp(string,P2(1))
      string='2. Lower right corner : y'
      CALL inp(string,P2(2))
      string='2. Lower right corner : z'
      CALL inp(string,P2(3))
      string='3. Upper left corner : x'
      CALL inp(string,P3(1))
      string='3. Upper left corner : y'
      CALL inp(string,P3(2))
      string='3. Upper left corner : z'
      CALL inp(string,P3(3))
C
1303  format (a,3(f9.5,1x))
304   write(*,*) 'File-name for your map ?'
      read(*,2010) filmap
      OPEN(20,FILE=filmap,STATUS='UNKNOWN',ERR=304)
      write(20,*) 'ix  iy   density'
      write(20,1303) 'Point1: ',P1(1), P1(2), P1(3)
      write(20,1303) 'Point2: ',P2(1), P2(2), P2(3)
      write(20,1303) 'Point3: ',P3(1), P3(2), P3(3)
      print *

      do 10 i=1,3
        D12(i)=P2(i)-P1(i)
        D13(i)=P3(i)-P1(i)
10    continue

C     base-transformation of triclinic to orthonormal coordinates:
C     (see Giacovazzo, 1992, Fund. of Cryst., p. 68) 
C     (x  y  z )= (M transposed) (x' y' z')         (x y z ) in tricl. coord.
C     (x' y' z')= (M transposed)^(-1) (x y z)       (x'y'z') in kart. coord.
C     using: ca cb cg= cos alpha, cos beta, cos gamma
C            a b c cell constants
C            a# b# c#, alpha#, beta#, gamma#  reciprocal cell constants
C            ca# cb# cg#= cos alpha#, cos beta#, cos gamma#
C     the matrices are
C     M = (1/a         0          0 )  M^(-1)=(a       0            0)
C         (-cg/(a sg)  1/(b sg)   0 )         (b cg    b sg         0)
C         (a# cb#      b# ca#     c#)         (c cb    -c sb ca#    1/c#)
C
C     the transposed M and M^(-1) are now called AM(,) and AMI(,)
C
C     the constants available from the subroutine PRECAL are
C     CELL(1,2,3)= a, b, c
C     as,bs,cs= a#, b#, c#
C     ca,cb,cg,sa,sb,sg= sin and cos of alpha,beta,gamma
C     ga, gb, gg = 2*cos(alpha#, beta#, gamma#)
C 
      AM(1,1) = 1/CELL(1)
      AM(2,1) = 0.
      AM(3,1) = 0.
      AM(1,2) = -cg/(CELL(1)*sg)
      AM(2,2) = 1/(CELL(2)*sg)
      AM(3,2) = 0.
      AM(1,3) = as*(gb/2)
      AM(2,3) = bs*(ga/2)
      AM(3,3) = cs
      AMI(1,1) = CELL(1)
      AMI(2,1) = 0.
      AMI(3,1) = 0.
      AMI(1,2) = CELL(2)*cg
      AMI(2,2) = CELL(2)*sg
      AMI(3,2) = 0.
      AMI(1,3) = CELL(3)*cb
      AMI(2,3) = -CELL(3)*sb*(ga/2)
      AMI(3,3) = (1/cs)

C     get points 1, 2 and 3 in new kartesian coordinates
      CALL MVmul(AMI,P1,UP1)
      CALL MVmul(AMI,P2,UP2)
      CALL MVmul(AMI,P3,UP3)

C     get centre of mass in new coordinates
      CM123(1)=(UP1(1)+UP2(1)+UP3(1))/3
      CM123(2)=(UP1(2)+UP2(2)+UP3(2))/3
      CM123(3)=(UP1(3)+UP2(3)+UP3(3))/3

C     get unit vectors D12  and D13 in new kartesian coordinates
      CALL MVmul(AMI,D12,TEMP)
      CALL VVequal(TEMP,D12)
      CALL Vunit(D12)
      CALL MVmul(AMI,D13,TEMP)
      CALL VVequal(TEMP,D13)
      CALL Vunit(D13)

C     get unit vector np = d12 x d13 normal to 12 and 13 in kart. coordinates
C     this is the plane normal vector
      CALL VVmul(D12,D13,NP)
      if (NP(1)**2+NP(2)**2+NP(3)**2.lt.1e-7) then
        print *,'Three points are collinear. Try again!'
        goto 4
      endif
      CALL Vunit(NP)

C     get vector nu = np x d12 normal to np and d12 in kartesian coordinates
C     this gives the normal coordinate-system (d12, nu) within the section plane
C     (using the old origin).
      CALL VVmul(NP,D12,NU)
      CALL Vunit(NU)

      write(*,*) 'Three given points in plane:'
      write(*,*) 'in cell (xyz) and in kart. coordinates (in Angstrom)'
      write(*,888) (P1(i), i=1,3),(UP1(i), i=1,3)
      write(*,888) (P2(i), i=1,3),(UP2(i), i=1,3)
      write(*,888) (P3(i), i=1,3),(UP3(i), i=1,3)
     
      print * 
      print *,'All coordinates are written into file ecken.txt'
      OPEN(21,FILE='ecken.txt',STATUS='UNKNOWN')
      write(21,*) 'Name of density file: ', filden
      write(21,*) '------------------------------'
      write(21,'(a)') title(1:80)
      write(21,'(a)') title(81:160)
      write(21,'(a)') title(161:200)
      write(21,*) '------------------------------'
888   FORMAT ('x=',f8.4,' y=',f8.4,' z=',f8.4, '  kx,ky,kz/Angstrom='
     $ ,3(f8.2,1X))
      write(21,*) 'Three given points in plane: in cell-(xyz) and in'
      write(21,*) 'kart. coordinates (in Angstrom)'
      write(21,888) (P1(i), i=1,3),(UP1(i), i=1,3)
      write(21,888) (P2(i), i=1,3),(UP2(i), i=1,3)
      write(21,888) (P3(i), i=1,3),(UP3(i), i=1,3)
      write(21,*) '(unit) difference vectors 12 and 13 '
      write(21,888) (P2(i)-P1(i),i=1,3),(D12(i), i=1,3) 
      write(21,888) (P3(i)-P1(i),i=1,3),(D13(i), i=1,3)
      write(21,*) 'plane normal vector in kart. coord.'
      CALL MVmul(AM,NP,TEMP) 
      write(21,888) (TEMP(i), i=1,3), (NP(i),i=1,3)
      write(21,*) 'map y-axix-vector in kart. coord.'
      CALL MVmul(AM,NU,TEMP) 
      write(21,888) (TEMP(i), i=1,3), (NU(i),i=1,3)
      write(21,*) 'center-of-mass coordinates: the center of your map'
      CALL MVmul(AM,CM123,TEMP) 
      write(21,888) (TEMP(i), i=1,3), (CM123(i),i=1,3)

      print *
      print *,'Where is the origin in your map:'
      print *,
     $ '0 = Center of your map will be the center of the'
     $     ,'    three points defining the plane.'
      print *,
     $ '1 = Point 1 will be at the lower left corner of your map.'
54    write(*,1002,ERR=54) 'Your choice ? ',imapor
      CALL abfrage(wert,i,CR,error)
      if (error) goto 54
      if (.not.CR) then
         if (i.gt.1 .or. i.lt.0) then
           print *,'wrong input'
           goto 54
        else 
          imapor=i
        endif
      endif
      print *,
     $  'Please input the length of your map edges in Angstrom:'
55    write(*,1001) 'x-axis edge along vector of point 1-->2 ?'
     $             ,maplx
      CALL abfrage(wert,i,CR,error)
      if (error) goto 55
      if (.not.CR) then
         if (wert.gt.1000 .or. wert.lt.0.01) then
           print *,'wrong input'
           goto 55
        else 
          maplx=wert
        endif
      endif

56    write(*,1001) 'y-axis edge perpendicular to vector 1-->2 ?'
     $             ,maply
      CALL abfrage(wert,i,CR,error)
      if (error) goto 56
      if (.not.CR) then
         if (wert.gt.1000 .or. wert.lt.0.01) then
           print *,'wrong input'
           goto 56
        else 
          maply=wert
        endif
      endif
      write(21,*) 'length of x-axis of map ',sngl(maplx),' Angstrom'
      write(21,*) 'length of y-axis of map ',sngl(maply),' Angstrom'
C     *** calculate lengths in unit-coordinate system (O, D12, NU)
C     *** vector (x=1,y=0) is one Angstrom long
57    write(*,1002) 'How many pixels along x-axis ? ',imappx
      CALL abfrage(wert,i,CR,error)
      if (error) goto 57
      if (.not.CR) then
         if (i.gt.1000 .or. i.lt.2) then
           print *,'wrong input'
           goto 57
        else 
          imappx=i
        endif
      endif

58    write(*,1002) 'How many pixels along y-axis ? ',imappy
      CALL abfrage(wert,i,CR,error)
      if (error) goto 58
      if (.not.CR) then
         if (i.gt.1000 .or. i.lt.2) then
           print *,'wrong input'
           goto 58
        else 
          imappy=i
        endif
      endif

      write(21,'(a,2i5)') 'pixels along x and y-axis:',imappx,imappy
      
105   print *,'Do you want a projection of density onto the plane?'
      print *,'(0=NO, 1=YES)'
      iproj=0
      CALL abfrage(wert,i,CR,error)
      if (error) goto 105
      if (.not.CR) then
        if (i.gt.1 .or. i.lt.0) then
          print *,'wrong input'
          goto 105
        else 
          iproj=i
        endif
      endif
      if (iproj.eq.1) then 
        print *,'Projection will be done. The projection range will be'
        print *,'given by two planes parallel to the current one.'
        print *,
     $ 'Please input the DISTANCE (in Angstrom) of the LOWER plane:'
        print *,'(If you input a negative value, this means that '
        print *,'your plane is below the chosen one)'
        read(*,*) projd1 
        print *,
     $ 'Please input the DISTANCE (in Angstrom) of the UPPER plane:'
        read(*,*) projd2 
        print *,'How many planes should be calculated for this ',
     $          'projection and be averaged ?'
        read(*,*) iprojn 
        if (iprojn.le.1) then
           print *,'No projection done!'
           iproj=0
        endif 
        projstep=(projd2-projd1)/(iprojn-1)
        if (iproj.eq.1) then
          write(21,*)
          write(21,*) '         PROJECTION ON PLANE!!!'
          write(21,*) 'Two distances of planes below and above plane:'
          write(21,*) sngl(projd1), sngl(projd2)
          write(21,'(i4,a)') iprojn,
     $       ' planes are averaged in this range.'
          write(21,*) 'projstep=',sngl(projstep)
          write(21,*)
        endif
      endif
      power=2.0
      smooth=0.0
      next=2
110   print *
      print *,'INTERPOLATION PARAMETERS (inverse distance)'
      print *
      print *,'value = SUM (weight*value) / SUM weights  '
      print *,'with weight = 1/(distance+smooth)^power'
      print *, '1 -- change power parameter:  now ',sngl(power)
      print *, '2 -- change smooth parameter: now ',sngl(smooth)
      print *, '3 -- change next parameter:   now ',next
      print *, '0 or RETURN -- continue without changes'
      read (*,'(BN,i4)') i
      if (i.lt.0 .or. i.gt.3) goto 110
      if (i.eq.1) then
        print *,'Give new value for power'
        read (*,*) power
        goto 110
      elseif (i.eq.2) then
        print *,'Give new value for smooth'
        read (*,*) smooth
        goto 110
      elseif (i.eq.3) then
        print *,'Give new value for next'
        read (*,*) next
        goto 110
      endif
      print *
      write(21,*) 'Interpolation parameters: power, smooth, next'
      write(21,*) sngl(power), sngl(smooth), next
      print *,'please wait ...'
      deltax=maplx/(imappx-1)
      deltay=maply/(imappy-1)
      if (imapor.eq.0) then
C       *** center of map = center of mass of P1,P2,P3 **
        START(1)=CM123(1)-(maplx/2)*D12(1) - (maply/2)*NU(1)
        START(2)=CM123(2)-(maplx/2)*D12(2) - (maply/2)*NU(2)
        START(3)=CM123(3)-(maplx/2)*D12(3) - (maply/2)*NU(3)
      else
C       *** P1 is lower left corner or map ***
        START(1)=UP1(1)
        START(2)=UP1(2)
        START(3)=UP1(3)
      endif
C
120   write(21,*) 'lower left edge of map'
      CALL MVmul(AM,START,TEMP) 
      write(21,888) (TEMP(i), i=1,3), (START(i),i=1,3)
C
      write(21,*) 'upper right edge of map'
      TEMP2(1)=START(1) + maplx*D12(1) + maply*NU(1)
      TEMP2(2)=START(2) + maplx*D12(2) + maply*NU(2)
      TEMP2(3)=START(3) + maplx*D12(3) + maply*NU(3)
      CALL MVmul(AM,TEMP2,TEMP) 
      write(21,888) (TEMP(i), i=1,3), (TEMP2(i),i=1,3)
C
      write(21,*) 'lower right edge of map'
      TEMP2(1)=START(1) + maplx*D12(1) 
      TEMP2(2)=START(2) + maplx*D12(2)
      TEMP2(3)=START(3) + maplx*D12(3)
      CALL MVmul(AM,TEMP2,TEMP) 
      write(21,888) (TEMP(i), i=1,3), (TEMP2(i),i=1,3)
C
      write(21,*) 'upper left edge of map'
      TEMP2(1)=START(1) +               maply*NU(1)
      TEMP2(2)=START(2) +               maply*NU(2)
      TEMP2(3)=START(3) +               maply*NU(3)
      CALL MVmul(AM,TEMP2,TEMP) 
      write(21,888) (TEMP(i), i=1,3), (TEMP2(i),i=1,3)
C
      do 210 iy=0,imappy-1
      do 210 ix=0,imappx-1
        UX(1)=START(1) + ix*deltax*D12(1) + iy*deltay*NU(1)
        UX(2)=START(2) + ix*deltax*D12(2) + iy*deltay*NU(2)
        UX(3)=START(3) + ix*deltax*D12(3) + iy*deltay*NU(3)
C       *** calculate cell-coordinates from kart. coord. via matrix AM
        if (iproj.eq.0) then
C         *** calculate this single point ***
          CALL MVmul(AM,UX,X) 
C         *** interpolation from a maxent-density array
          CALL INTER(mpix,NPIX,DELC,power,smooth,next,X,density)
        else
C         *** calculate projection average over line perpendicular to plane ***
          densum=0
          do 208 ip=0,iprojn-1
            TEMP(1)=UX(1) + (ip*projstep + projd1)*NP(1)
            TEMP(2)=UX(2) + (ip*projstep + projd1)*NP(2)
            TEMP(3)=UX(3) + (ip*projstep + projd1)*NP(3)
            CALL MVmul(AM,TEMP,X) 
C           *** interpolation from a maxent-density array
            CALL INTER(mpix,NPIX,DELC,power,smooth,next,X,density)
            densum=densum+density
208       continue
          density=densum/iprojn
        endif
        write(20,'(i4,1x,i4,2x,G21.5)') ix,iy,density
C       write(20,*) ix,iy,sngl(density)
210   continue
      close(20)
      print *,'Do you like another plane, parallel to the one'
     $        ,' just calculated ? '
      print *,'(0=NO, 1=YES)'
      read (*,2000) i
      if (i.eq.0) then
        close(21) 
        RETURN
      endif
C
C     *** another plane ***
225   write(*,*) 'Please give distance (pos. or neg.) in Agstrom'
      read (*,*) wert        
      if (wert.lt.-1000 .or. wert.gt.1000 .or. wert.eq.0) goto 225
      write(21,*)
      write(21,*) '------------------------------------------'
      write(21,*) 'New plane parallel in distance  ',sngl(wert) 
      write(21,*)
      START(1)=START(1) + wert*NP(1) 
      START(2)=START(2) + wert*NP(2)
      START(3)=START(3) + wert*NP(3)
305   write(*,*) 'File-name for your map ?'
      read(*,2010) filmap
      OPEN(20,FILE=filmap,STATUS='UNKNOWN',ERR=305)
      goto 120 
      end
C========================================================================
C
C     ------------------------
      SUBROUTINE MVmul(AM,V,U)
C     *** calculate V = M V with M a 3x3 matrix, and V a 3x1 vector
C 
      implicit none
      real*8 AM(3,3), V(3), U(3)
      U(1) = V(1)*AM(1,1) + V(2)*AM(1,2) + V(3)*AM(1,3)
      U(2) = V(1)*AM(2,1) + V(2)*AM(2,2) + V(3)*AM(2,3)
      U(3) = V(1)*AM(3,1) + V(2)*AM(3,2) + V(3)*AM(3,3)
      return
      end
C
C     ------------------------
      SUBROUTINE Vunit(V)
C     *** make V a unit 3x1 vector
C 
      implicit none
      real*8 V(3), btr
      btr    = SQRT(V(1)*V(1) + V(2)*V(2) + V(3)*V(3))
      V(1)  = V(1)/btr
      V(2)  = V(2)/btr
      V(3)  = V(3)/btr
      return
      end
C
C     ------------------------
      SUBROUTINE VVmul(V,W,R)
C     *** multiply two 3x1 vectors V1 x V2 --> R
C 
      implicit none
      real*8 V(3), W(3), R(3)
      R(1) = V(2)*W(3) - V(3)*W(2)
      R(2) = V(3)*W(1) - V(1)*W(3)
      R(3) = V(1)*W(2) - V(2)*W(1)
      return
      end
C
C     ------------------------
      SUBROUTINE VVequal(V,W)
C     *** give the 3x1 vector W the value of V
C 
      implicit none
      real*8 V(3), W(3)
      W(1) = V(1)
      W(2) = V(2)
      W(3) = V(3)
      return
      end
C
C     ------------------------
      SUBROUTINE VVadd(V,W,U)
C     *** U = V + W
C 
      implicit none
      real*8 V(3), W(3), U(3)
      U(1) = V(1) + W(1)
      U(2) = V(2) + W(2)
      U(3) = V(3) + W(3)
      return
      end
C     ------------------------
      SUBROUTINE VSmul(V,s,U)
C     *** U = s*V     s is skalar, V vector
C 
      implicit none
      real*8 s
      real*8 V(3), U(3)
      U(1) = s*V(1)
      U(2) = s*V(2)
      U(3) = s*V(3)
      return
      end
C     ------------------------
      SUBROUTINE INTER(mpix,NPIX,DELC,power,smooth,next,X,density)
C     *** interpolate the density ARRAY DELC with discrete density to
C     *** get the density in point X(x,y,z) (relative coordinates).
C     *** interpolation is done with the method of inverse distance
C     *** CAUTION: in this routine, interpolation is done in the pixel
C     *** space only. If the pixel shape in real space is very different 
C     *** from a cube with equal edges, then this routine will give an 
C     *** imperfect interpolation.
C     *** For `inverse square interpolation' this equation is used:
C
C       Z = ( sum_i Z_i/((d_i+smooth)^power)/ sum_i 1/((d_i+smooth)^power) )
C     
C     with Z the resulting value, Z_i the densities around the point X,
C     and d_i the distance in pixel space.
C
C     usually 1<=power<=3, and smooth=0.
C     the next parameter is the maximum pixel number distance between
C     the point X and the neighbours used in the interpolation: next >= 1.
C
      IMPLICIT  none
      integer NPIX(3),mpix,nx0,ny0,nz0,next
C     *** DELC(0..NPIX(1), 0.., 0..) contains the whole unit cell density
      real*8 DELC(0:mpix,0:mpix,0:mpix)
      real*8 X(3),AX(3),density,power,smooth,sumwei,sumZ,dist,weight
      integer ni,nj,nk,i,j,k
C      
      X(1) = DMOD(20.+X(1),1d0)
      X(2) = DMOD(20.+X(2),1d0)
      X(3) = DMOD(20.+X(3),1d0)
      ax(1)=X(1)*NPIX(1)
      ax(2)=X(2)*NPIX(2)
      ax(3)=X(3)*NPIX(3)
      nx0 = INT(ax(1))
      ny0 = INT(ax(2))
      nz0 = INT(ax(3))
      if (next.lt.1) next=1
      sumwei=0
      sumZ  =0
      do 10 i=nx0-next+1,nx0+next
      do 10 j=ny0-next+1,ny0+next
      do 10 k=nz0-next+1,nz0+next
        ni=mod(2*NPIX(1)+i,NPIX(1))
        nj=mod(2*NPIX(2)+j,NPIX(2))
        nk=mod(2*NPIX(3)+k,NPIX(3))
        dist=(ax(1)-i)**2 + (ax(2)-j)**2 +(ax(3)-k)**2   
        if (dist.le.0.0025) then
C         *** if distance is < 1/20 pixel, use its value instead of 
C         *** an interpolation value
          density=DELC(ni,nj,nk)
          goto 11
        else 
          weight=1/((dist+smooth)**power)
          sumwei=sumwei+weight
          sumZ=sumZ + DELC(ni,nj,nk)*weight
        endif 
10    continue
      density = sumZ/sumwei
11    return
      end
C     ---------------------------
      SUBROUTINE inp(string,value)
C     *** input a real value with default ***
      real*8 value,wert
      character*80 string
      integer i
      logical CR,error
1001  FORMAT(a,'  [default=',f8.5,']')
13    write(*,1001) string,value
      CALL abfrage(wert,i,CR,error)
      if (error) goto 13
      if (.not.CR) then
         if (wert.gt.2 .or. wert.lt.-2) then
           print *,'wrong input'
           goto 13
        else 
          value=wert
        endif
      endif
      return
      end

